<?php

/***************************************************************
*  Copyright notice
*
*  (c) 2008-2013 Joachim Ruhs <postmaster@joachim-ruhs.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Plugin 'Locator' for the 'locator' extension.
 *
 * @author        Joachim Ruhs <postmaster@joachim-ruhs.de>
 */

if (!class_exists('tslib_pibase')) require_once (PATH_tslib . 'class.tslib_pibase.php');
//require_once (PATH_t3lib . 'class.t3lib_userauthgroup.php');

class tx_locator_model extends tslib_pibase {

	/* Global Configuration */
	var $conf;

	/* Set Globals Configuration ... */
	function setConfiguration($conf) {
		$this->conf = $conf;
	}

	function setLocalLang(& $LOCAL_LANG) {
		$this->LOCAL_LANG = $LOCAL_LANG;
	}

	/* Default Setters & Getters */
	function getUid() {
		if ($this->uid === NULL)
			$this->refresh();
		return $this->uid;
	}
	function setUid($id) {
		$this->uid = $id;
	}

	function getPid() {
		if ($this->uid === NULL)
			$this->refresh();
		return $this->pid;
	}

	function setGPVars($gp) {
		$this->_GP = $gp;
		$this->pi_loadLL();
	}

	function after($this1, $inthat) {
		if (!is_bool(strpos($inthat, $this1)))
			return substr($inthat, strpos($inthat, $this1) + strlen($this1));
	}

	function after_last($this1, $inthat) {
		if (!is_bool(strrevpos($inthat, $this1)))
			return substr($inthat, strrevpos($inthat, $this1) + strlen($this1));
	}

	function before($this1, $inthat) {
		return substr($inthat, 0, strpos($inthat, $this1));
	}

	function before_last($this1, $inthat) {
		return substr($inthat, 0, strrevpos($inthat, $this1));
	}

	function between($this1, $that, $inthat) {
		return $this->before($that, $this->after($this1, $inthat));
	}

	function between_last($this1, $that, $inthat) {
		return $this->after_last($this1, $this->before_last($that, $inthat));
	}

	function getStrsBetween($s, $s1, $s2 = false, $offset = 0) {
		/*====================================================================
		Function to scan a string for items encapsulated within a pair of tags

		getStrsBetween(string, tag1, <tag2>, <offset>

		If no second tag is specified, then match between identical tags

		Returns an array indexed with the encapsulated text, which is in turn
		a sub-array, containing the position of each item.

  		Notes:
		strpos($needle,$haystack,$offset)
		substr($string,$start,$length)

		====================================================================*/

		if ($s2 === false) {
			$s2 = $s1;
		}
		$result = array ();
		$L1 = strlen($s1);
		$L2 = strlen($s2);

		if ($L1 == 0 || $L2 == 0) {
			return false;
		}

		do {
			$pos1 = strpos($s, $s1, $offset);

			if ($pos1 !== false) {
				$pos1 += $L1;

				$pos2 = strpos($s, $s2, $pos1);

				if ($pos2 !== false) {
					$key_len = $pos2 - $pos1;

					$this_key = substr($s, $pos1, $key_len);

					if (!array_key_exists($this_key, $result)) {
						$result[$this_key] = array ();
					}

					$result[$this_key][] = $pos1;

					$offset = $pos2 + $L2;
				} else {
					$pos1 = false;
				}
			}
		} while ($pos1 !== false);
		return $result;
	}

	function strrevpos($instr, $needle) {
		$rev_pos = strpos(strrev($instr), strrev($needle));
		if ($rev_pos === false)
			return false;
		else
			return strlen($instr) - $rev_pos -strlen($needle);
	}

	function get_webpage($url) {
		//global $db;
		if (ini_get('allow_url_fopen'))
			$this->conf['useCurl'] = 0;
		else
			$this->conf['useCurl'] = 1;

		if ($this->conf['useCurl']) {
			$sessions = curl_init();
			curl_setopt($sessions, CURLOPT_URL, $url);
			curl_setopt($sessions, CURLOPT_HEADER, 0);
			curl_setopt($sessions, CURLOPT_RETURNTRANSFER, 1);
			$data = curl_exec($sessions);
			curl_close($sessions);
		} else {
			 //ini_set('allow_url_fopen','on');
			//$data = @ file_get_contents($url);
			$data = t3lib_div::getUrl($url);
		}
		return $data;
	}

	function geocodeAddress($complete_address /*, &$start=null*/) {
		//$err = error_reporting(0);
		//This is where we add different geocoding sites for each country.
		//Currently supported:
		//US: Zipcode and Addresses
		//CA: Zipcode and Addresses
		//UK: Postal Code and Addresses
		//International: City names with Country

		//Test Addresses for debugging purposes and accuracy testing
		//US: 7997 S Brook Forest Rd, Evergreen, CO 80439 US
		//CA: 10115 97a Ave, Edmonton, AB T6E4T2 CA
		//CA: 180 boulevard Omer-Marcil ,Saint-Jean-sur-Richelieu,QC CA
		//UK: 48 Leicester Square, London UK SW11
		//AU: 201 sussex, sydney, NSW au
		//NZ: 1200 glenmore st, Wellington, NZ
		//International: Tokyo, JP

		//Replace spaces in the address with + signs to be used in the urls.  Most, if not all geocoders will support
		//+ symbols as spaces since it's a standard URI symbol for space
		//Addresses come in as an array parse it out.


		//for urlencoding
		$vars = array (
			'zipcode',
			'city',
			'address',
			'country'
		);
		while (list (, $v) = each($vars)) {
			$complete_address[$v] = urlencode($complete_address[$v]);
		}
		$address = $complete_address['address'];
		$city = $complete_address['city'];
		$state = $complete_address['state'];
		$country = $complete_address['country'];
		$zipcode = $complete_address['zipcode'];

		if ($GLOBALS['TSFE']->localeCharset == 'utf-8')
			$city = mb_strtolower($city, "UTF-8"); // especially used for the polish dashed L

		######################################Main Geocoders#####################################
		if (!$lat_lon->lat && !$lat_lon->lon) {
//			$apiURL = "http://maps.googleapis.com/maps/api/geocode/xml?address=$address,+$zipcode+$city,+$country&sensor=false";

			$apiURL = "http://maps.googleapis.com/maps/api/geocode/json?address=$address,+$zipcode+$city,+$country&sensor=false";
			$addressData = $this->get_webpage($apiURL);


			$coordinates[1] = json_decode($addressData)->results[0]->geometry->location->lat;
			$coordinates[0] = json_decode($addressData)->results[0]->geometry->location->lng;


			$lat_lon = new \stdclass();
			$lat_lon->lat = $coordinates[1];
			$lat_lon->lon = $coordinates[0];
			$lat_lon->status = json_decode($addressData)->status;
			if ($this->conf['debug']){
				    $devLog = array ( '$addressData' => $addressData);
					t3lib_div::devLog('geocodeAddress', 'tx_locator', 3, $devLog);
					t3lib_div::devLog('coordinates', 'tx_locator', 3, $coordinates);

			}

		}

		####################################International########################################

		//If there is no lon/lat yet, we assume it's international and attempt to find it based on just the
		//city and country.
		if (!$lat_lon->lat && !$lat_lon->lon) {
			$apiURL = "http://maps.googleapis.com/maps/api/geocode/json?address=$city,+$country&sensor=false";

//			$apiURL = utf8_encode($apiURL);
			$addressData = $this->get_webpage($apiURL);

			$coordinates[1] = json_decode($addressData)->results[0]->geometry->location->lat;
			$coordinates[0] = json_decode($addressData)->results[0]->geometry->location->lng;

			$latLon = new \stdclass();
			$lat_lon->lat = $coordinates[1];
			$lat_lon->lon = $coordinates[0];
			$lat_lon->status = json_decode($addressData)->status;
			if ($this->conf['debug'])
				    $devLog = array ( 'google city and country' => $addressData);
					t3lib_div::devLog('geocodeAddress', 'tx_locator', 3, $devLog);
		}
		return $lat_lon;
	}

	function update_lat_lon() {
		// sanitizing when called from eID-script
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';


		if (!$this->conf['updateLatLonLimit']) $this->conf['updateLatLonLimit'] = 20;

		$this->_GP['update'] = true; // switch for geocodeAddress
		if ($this->conf['useFeUserData'] == 1) {
			$table = 'fe_users';
			$fields = "*, zip as zipcode, country as state";
			$where_clause = " tx_locator_lat = '' AND tx_locator_lon = '' AND tx_locator_geocode = 1 AND pid IN (" . $this->conf['pid_list'] .
			") AND (" . intval($this->conf['feUserGroup']) . " in (usergroup) OR " .
			" usergroup like '%," . intval($this->conf['feUserGroup']) . "' OR " .
			" usergroup like '%," . intval($this->conf['feUserGroup']) . ",%')" .

			" AND deleted='0' AND disable='0'";
		}
		if ($this->conf['useFeUserData'] == 0) {
			$table = 'tx_locator_locations';
			$fields = "*";
			$where_clause = " lat = '' AND lon = '' AND geocode = 1  AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
		}
		if ($this->conf['useFeUserData'] == 2) {
			$table = $this->conf['externalLocationTable'];
			$fields = "*, street as address, zip as zipcode, country as state";
			$where_clause = " ((tx_locator_lat IS NULL AND tx_locator_lon IS NULL) OR (tx_locator_lat = '' AND tx_locator_lon = '')) AND tx_locator_geocode = 1 AND pid IN (" . $this->conf['pid_list'] .
			")" .
			" AND deleted='0' ";
		}
		if ($this->conf['useFeUserData'] == 3) {
			$table = 'tt_news';
			$fields = "*, title as storename, tx_locator_address as address, tx_locator_zipcode as zipcode, tx_locator_city as city," .
			" tx_locator_country as country, tx_locator_state as state";
			$where_clause = " ((tx_locator_lat IS NULL AND tx_locator_lon IS NULL) OR (tx_locator_lat = '' AND tx_locator_lon = '')) AND tx_locator_geocode = 1 AND pid IN (" . $this->conf['pid_list'] .
			")" .
			" AND deleted='0' ";
		}

		// martin start
		if ($this->conf['useFeUserData'] == 4) {
			$table = 'tt_address';
			$fields = "*, company as storename, address, zip as zipcode, city," .
			" country, region as state";
			$where_clause = " ((tx_locator_lat IS NULL AND tx_locator_lon IS NULL) OR (tx_locator_lat = '' AND tx_locator_lon = '')) AND tx_locator_geocode = 1 AND pid IN (" . $this->conf['pid_list'] .
			")" .
			" AND deleted='0' ";
		}
		// martin stop

		$orderBy = '';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where_clause, '', $orderBy, intval($this->conf['updateLatLonLimit']));
		$fields = array ();

		while ($rows = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			if ($rows['uid']) {
				$zipcode = $rows['zipcode'];
				$complete_address['address'] = $rows['address'];
				$complete_address['city'] = $rows['city'];
				$complete_address['state'] = $rows['state'];
				$complete_address['zipcode'] = $rows['zipcode'];
				$complete_address['country'] = $rows['country'];

				if ($this->conf['useFeUserData'] == 1) {
					$fields['tx_locator_geocode'] = 1;
				}
				if ($this->conf['useFeUserData'] == 0) {
					$fields['geocode'] = 1;
				}
				if ($this->conf['useFeUserData'] == 2) {
					$fields['tx_locator_geocode'] = 1;
				}
				if ($this->conf['useFeUserData'] == 3) {
					$fields['tx_locator_geocode'] = 1;
				}
				//martin start
				if ($this->conf['useFeUserData'] == 4) {
					$fields['tx_locator_geocode'] = 1;
				}
				//martin stop

				$lat_lon = $this->geocodeAddress($complete_address);
				if ($lat_lon->lat == null) {
					if ($this->conf['useFeUserData'] == 1) {
						$fields['tx_locator_geocode'] = 0;
						$message = 'Could not geocode ' . $rows['name'] . ' ' . $rows['company'] . ' ' . $rows['address'] . ' ' . $rows['zipcode'] . ' ' . $rows['city'];
					}
					if ($this->conf['useFeUserData'] == 0) {
						$fields['geocode'] = 0;
						$message = 'Could not geocode ' . $rows['storename'] . ' ' . $rows['address'] . ' ' . $rows['zipcode'] . ' ' . $rows['city'];
					}
					if ($this->conf['useFeUserData'] == 2) {
						$fields['tx_locator_geocode'] = 0;
						$message = 'Could not geocode ' . $rows['name'] . ' ' . $rows['company'] . ' ' . $rows['address'] . ' ' . $rows['zipcode'] . ' ' . $rows['city'];
					}
					if ($this->conf['useFeUserData'] == 3) {
						$fields['tx_locator_geocode'] = 0;
						$message = 'Could not geocode ' . $rows['name'] . ' ' . $rows['company'] . ' ' . $rows['address'] . ' ' . $rows['zipcode'] . ' ' . $rows['city'];
					}
					//martin start
					if ($this->conf['useFeUserData'] == 4) {
						$fields['tx_locator_geocode'] = 0;
						$message = 'Could not geocode ' . $rows['name'] . ' ' . $rows['company'] . ' ' . $rows['address'] . ' ' . $rows['zipcode'] . ' ' . $rows['city'];
					}
					//martin stop

					if ($this->conf['enableLogs']) {
						$this->logUser = t3lib_div :: makeInstance("t3lib_beUserAuth");
						$this->logUser->simplelog($message, 'tx_locator', $error = 2);
					}
				}

				$lat = (float) $lat_lon->lat;
				$lon = (float) $lat_lon->lon;
				$uid = (int) $rows['uid'];

				if ($this->conf['useFeUserData'] == 0) {
					$fields["lat"] = $lat;
					$fields["lon"] = $lon;
				}
				if ($this->conf['useFeUserData'] == 1) {
					$fields["tx_locator_lat"] = $lat;
					$fields["tx_locator_lon"] = $lon;
				}
				if ($this->conf['useFeUserData'] == 2) {
					$fields["tx_locator_lat"] = $lat;
					$fields["tx_locator_lon"] = $lon;
				}
				if ($this->conf['useFeUserData'] == 3) {
					$fields["tx_locator_lat"] = $lat;
					$fields["tx_locator_lon"] = $lon;
				}

				//martin start
				if ($this->conf['useFeUserData'] == 4) {
					$fields["tx_locator_lat"] = $lat;
					$fields["tx_locator_lon"] = $lon;
				}
				//martin stop

				$where = "uid=" . $uid;
				// update only when request was successful
				// this is needed because when google limit is reached the records should not be udated
				// status would be OVER_QUERY_LIMIT
				if ($lat_lon->status == 'OK' || $lat_lon->status == 'ZERO_RESULTS')
					$GLOBALS['TYPO3_DB']->exec_UPDATEquery($table, $where, $fields);

				// fetch zipcode area
				if ($this->conf['shapeFile'] == '' && $this->conf['displayMode'] == 'zipcodeAreaView') {
					if ($this->conf['zipcodeAreaUrl'] != '') {
						$url = $this->conf['zipcodeAreaUrl'] . $complete_address['zipcode'];
						$data = @ file_get_contents($url);
						// may be more than 1 area
						$data = explode('map.addOverlay', $data);
						$this->insertZipcodeArea($complete_address['zipcode'], $uid, $data);
					}
				}
				if ($this->conf['shapeFile'] != '' && $this->conf['displayMode'] == 'zipcodeAreaView') {
					if ($this->conf['displayMode'] == 'zipcodeAreaView') {
						require_once (PATH_site . 'typo3conf/ext/locator/pi1/helper/getShape.php');
					}
				}
			}
		}
		/* update object information */
		$this->_GP['update'] = false;

	}

	function insertZipcodeArea($zipcode, $storeUid, $data) {
		// sanitizing
		// only Germany 0-9
		if (preg_match('/^[0-9 ]*$/', $zipcode) == 1) {
		    // all is ok
		} else $zipcode = '';
		$table = 'tx_locator_zipcodeareas';
		$where = 'zipcode=' . $zipcode;
		$result = $GLOBALS['TYPO3_DB']->exec_DELETEquery($table, $where);

		if ($this->conf['shapeFile'] != '' && $this->conf['displayMode'] == 'zipcodeAreaView') {

			for ($i = 0; $i < count($data['area']); $i++) {
				$fields = array (
					'pid' => $this->conf['pid_list'],
					'zipcode' => $zipcode,
					'storeuid' => $storeUid,
					'area' => $data['area'][$i],
					'lon' => $data['lon'][$i],
					'lat' => $data['lat'][$i],
				'crdate' => time(), 'cruser_id' => $GLOBALS[TSFE]->beUserLogin, 'tstamp' => time(),);
				$result = $GLOBALS['TYPO3_DB']->exec_INSERTquery($table, $fields);

			}
		}

		if ($this->conf['shapeFile'] == '' && $this->conf['displayMode'] == 'zipcodeAreaView') {
			for ($area = 0; $area < count($data); $area++) {

				$geodata = $this->getStrsBetween($data[$area], 'points.push(new GPoint(', '));');
				$keys = array_keys($geodata);

				for ($i = 0; $i < count($keys); $i++) {
					$arr = explode(',', $keys[$i]);
					$fields = array (
						'pid' => $this->conf['pid_list'],
						'zipcode' => $zipcode,
						'storeuid' => intval($storeUid),
						'area' => $area,
						'lon' => $arr[0],
						'lat' => $arr[1],
					'crdate' => time(), 'cruser_id' => $GLOBALS[TSFE]->beUserLogin, 'tstamp' => time(),);
					$result = $GLOBALS['TYPO3_DB']->exec_INSERTquery($table, $fields);

				}
			}
		}

	}

	function getTtAdressGroups() {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		$table = 'tt_address_group';
		$fields = '*';
		$where = " pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
		$orderBy = 'sorting';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, '', $orderBy);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$data['uid'][] .= $row['uid'];
			$data['category'][] .= htmlspecialchars($row['title']);
		}
		return $data;
	}
	function getFeUserCategories() {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		$table = 'fe_groups';
		$fields = '*';
		$where = " pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
		$orderBy = 'title';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, '', $orderBy);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$data['uid'][] .= $row['uid'];
			$data['category'][] .= htmlspecialchars($row['title']);
		}
		return $data;
	}

	function getCategories() {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		$table = 'tx_locator_categories';
		$fields = '*';
		$where = " pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
		$orderBy = 'sorting';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, '', $orderBy);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$data['uid'][] .= $row['uid'];
			$data['category'][] .= htmlspecialchars($row['name']);
		}
		return $data;
	}


	function getCatMenuCategories() {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		$this->cObj = t3lib_div :: makeInstance('tslib_cObj');
		$table = 'tx_locator_categories';
		$fields = '*';
		$where = " parentUid=0 AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
		$orderBy = 'sorting';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, '', $orderBy);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$data['uid'][] .= $row['uid'];
			$data['name'][] .= htmlspecialchars($row['name']);
		}
		$out .= '<div class="catMenu"><ul>';
		for ($i = 0; $i < count($data['uid']); $i++) {
			$urlParams = array (
				'tx_locator_pi1[categories]' => $data['uid'][$i],
					'tx_locator_pi1[address]' => $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['address']),
					'tx_locator_pi1[zipcode]' => $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['zipcode']),
					'tx_locator_pi1[city]' => $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['city']),
					'tx_locator_pi1[country]' => $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['country']),
		    );
		    $out .= '<li>' . $this->pi_linkToPage($data['name'][$i], $this->conf['resultPageId'], '_top', $urlParams);
		    $out .= $this->getCatMenuChildrenCategories($data['uid'][$i], '');
		    $out .= '</li>';
		}
		$out .= '</ul></div>';
		return $out;
	}
	
	function getCatMenuChildrenCategories($uid, $out) {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		$table = 'tx_locator_categories';
		$fields = '*';
		$where = ' parentUid=' . intval($uid) . ' AND pid IN (' . $this->conf['pid_list'] . ') AND deleted=0 AND hidden=0';
		$orderBy = 'sorting';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, '', $orderBy);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$data['uid'][] .= $row['uid'];
			$data['name'][] .= htmlspecialchars($row['name']);
		}
		if ($data) {
			$out .= '<ul>';
			for ($i = 0; $i < count($data['uid']); $i++) {
				$urlParams = array (
					'tx_locator_pi1[categories]' => $data['uid'][$i],
					'tx_locator_pi1[address]' => htmlspecialchars($this->_GP['address']),
					'tx_locator_pi1[zipcode]' => htmlspecialchars($this->_GP['zipcode']),
					'tx_locator_pi1[city]' => htmlspecialchars($this->_GP['city']),
					'tx_locator_pi1[country]' => htmlspecialchars($this->_GP['country']),
			    );
	    
			    $out .= '<li>' . $this->pi_linkToPage($data['name'][$i], $this->conf['resultPageId'], '_top', $urlParams);
			    $out .= '</li>';
			}
			$out .= '</ul>';
		}
		return $out;
	}

	function getChildCategories($cat) {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		if ($this->conf['useFeUserData'] == 4) {
			$table = 'tt_address_group';
			$where_clause = " parent_group = $cat AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
		}
		if ($this->conf['useFeUserData'] == 3) {
			$table = 'tt_news_cat';
			$where_clause = " parent_category = $cat AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
		} else {
			$table = 'tx_locator_categories';
//			$where_clause = " parentUid = $cat AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
			$where_clause = " parentUid in ($cat) AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
		}
		$fields = "*";
		$orderBy = 'sorting';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where_clause, '', $orderBy);
		$out = array ();
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$this->children[] .= $row['uid'];
			if ($row['uid'] > 0)
				$this->getChildCategories($row['uid']);

		}
		return $this->children;
	}

	function getCategory($categoryUid) {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		$table = 'tx_locator_categories';
		global $_EXTCONF, $TYPO3_CONF_VARS;
		$fields = "*";
		$where = " pid IN (" . $this->conf['pid_list'] . ") AND uid=" . intval($categoryUid) . " AND deleted='0' AND hidden='0'";
		$orderBy = 'sorting';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, '', $orderBy);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$name = htmlspecialchars($row['name']);
		}
		return $name;
	}

	function get_categories() {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		if ($GLOBALS['TSFE']->config['config']['sys_language_uid'] == '')
			$GLOBALS['TSFE']->config['config']['sys_language_uid'] = 0;

		if ($this->conf['useFeUserData'] == 3) { // tt_news table
			$table = 'tt_news_cat';
			$fields = "uid, title as name";
			$where_clause = " 1 AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
			$orderBy = 'uid';
			$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where_clause, '', $orderBy);
		} else {
			$table = 'tx_locator_categories';
			global $_EXTCONF, $TYPO3_CONF_VARS;
			$fields = "*";
			$where_clause = " pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";

			$where_clause .= ' AND ' . $GLOBALS['TSFE']->config['config']['sys_language_uid'] . '=sys_language_uid';

			$orderBy = 'sorting';
			$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where_clause, '', $orderBy);
		}

		$out = array ();
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$out['uid'][] .= $row['uid'];
			$out['category'][] .= htmlspecialchars($row['name']);
		}
		return $out;
	}

	function getBranches() {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		$table = 'tx_locator_locations';
		global $_EXTCONF, $TYPO3_CONF_VARS;
		$fields = "distinct tx_locatorparking_branch as branch";
		$where_clause = " pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
		$orderBy = 'sorting';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where_clause, '', $orderBy);
		$out = array ();
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			//$out['uid'][]         .= $row['uid'];
			$out['branch'][] .= htmlspecialchars($row['branch']);
		}
		return $out;
	}
	function getNumberOfStores() {
		global $_EXTCONF, $TYPO3_CONF_VARS;

		// sanitizing when called from eID-script
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		//martin start
		if ($this->conf['useFeUserData'] == 4) { // tt_address
			$fields = "count(*) as number";
			$where_clause = " tx_locator_lat != 0 AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
			//$orderBy = 'sorting';
			$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, 'tt_address', $where_clause, '', '');
		}
		//martin stop
		if ($this->conf['useFeUserData'] == 3) { // tt_news
			$fields = "count(*) as number";
			$where_clause = " tx_locator_lat != 0 AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
			//$orderBy = 'sorting';
			$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields,
													 'tt_news',
													 $where_clause,
													 '',
													 '');

		} else {
			$fields = "count(*) as number";
			$where_clause = " pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
			//$orderBy = 'sorting';
			$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, 'tx_locator_locations', $where_clause, '', '');

		}
		$out = array ();
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$num = $row['number'];
		}
		return $num;
	}

	function getL10nParentCategory($uid) {
		if ($uid == 0)
			return 0;
		$GLOBALS['TYPO3_DB']->debugOutput = 1;
		$fields = 'sys_language_uid, l10n_parent';
		$table = 'tx_locator_categories';
		$where = 'uid =' . intval($uid);
		$orderBy = '';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, '', $orderBy);
		$row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result);
		if ($row['sys_language_uid'] != 0) {
			// return the uid of original language
			return $row['l10n_parent'];
		}
		return $uid;
	}

	function inradius($complete_address, $radius) {
		global $TYPO3_CONF_VARS;
		$_EXTCONF = unserialize($TYPO3_CONF_VARS['EXT']['extConf']['locator']);

		// sanitizing when called from eID-script
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';


		if ($radius == 0)
			$radius = 1; // needed if 2 stores are nearly on same location
		if ($this->conf['distanceUnit'] == 'miles')
			$radius = $radius * 1.6;

		$radius = (float) $radius;

		//If there are any limitations on the results set, get them here.
		$results_limit = $this->conf['resultLimit'];

		if ($results_limit > 0) {
			$results_limit = "0,$results_limit";
		}

		//debug($complete_address,'complete');
		//Check to see if we already have starting lat/lon so we dont do double geocoding
		// attention: $complete_address['lat_lon'] is of type stdClass
		if (!$complete_address['lat_lon']->lat && !$complete_address['lat_lon']->lon) {
			$lat_lon = $this->geocodeAddress($complete_address);
		} else {
			$lat_lon = $complete_address['lat_lon'];
		}
		$lat = (float) trim($lat_lon->lat);
		$lon = (float) trim($lat_lon->lon);

		if ($lat == '')
			return; // no coordinates found

		// when categories are set by Typoscript use them
		if ($this->conf['categories']) {
			$complete_address['categories'] = $this->conf['categories'];
		}

//		if ($this->_GP['category'])
//		    $categories = $this->_GP['category'];


		//Add in categories to the search criteria
		//If they are not an array, make them an array.
		$categories = $complete_address['categories'];

		if (!is_array($categories)) {
			$categories = explode(',', $categories);
		}
		// when calendar base is used change fieldname
		if ($this->conf['useFeUserData'] == 2)
			$categoryField = 'tx_locator_categoryuid';
		else
			if ($this->conf['useFeUserData'] == 3)
				$categoryField = 'category';
			else
				$categoryField = 'categoryuid';


		// here we have to fetch all subcategories if category is a parent
		// and we use locator_categories
		if ($this->conf['useFeUserData'] == 0 || $this->conf['useFeUserData'] == 4 || $this->conf['useFeUserData'] == 3) {
			foreach ($categories as $cat) {
				$cat = (int) trim($cat);
				if ($cat >= 0)
					$addCat = $this->getChildCategories($cat);
			}

			for ($i = 0; $i < count($addCat); $i++)
				$categories[] = $addCat[$i];
			if ($this->conf['useFeUserData'] == 0) {
				// get the correct categoryUid for the default language
				for ($i = 0; $i < count($categories); $i++) {
					$categories[$i] = $this->getL10nParentCategory($categories[$i]);
				}
			}


			// sanitizing
			if (preg_match('/^[A-z,]*$/', $this->_GP['products']) == 1) {
			    // all is ok
			} else $this->_GP['products'] = '';

			$products = explode(',', $this->_GP['products']);
			if (count($products)) {
				$products_search = ' AND ( (';
				for ($i = 0; $i < count($products); $i++) {
					$products_search .= ($i != 0) ? ' OR ' : '';
					$products_search .= 'products like "%' . trim(strtoupper($products[$i])) . '%"';
				}
				// search in categorynames too
				$products_search .= ' ) OR (';
				for ($i = 0; $i < count($products); $i++) {
					$products_search .= ($i != 0) ? ' OR ' : '';

					$products_search .= 'b.name like "%' . trim(strtoupper($products[$i])) . '%"';
				}
				$products_search .= '))';
			}
		}



		reset($categories);
		if ($categories[0] == '')
			unset ($categories[0]);
		//Loop through and add each category to the query.


		$category_search = "AND (";
		if (count($categories) == 1) {
			if (!$categories[0]) {
				array_shift($categories);
			}

			$cat = (int) $categories[0]; // [0] -> [1] 13.01.2012    [1] -> [0] 29.02.2012
			$category_search .= " " . $categoryField . " = '$cat' ";
			$category_search .= " OR " . $categoryField . " LIKE '$cat,%' ";
			$category_search .= " OR " . $categoryField . " LIKE '%,$cat,%' ";
			$category_search .= " OR " . $categoryField . " LIKE '%,$cat' ";
		} else {
			foreach ($categories as $cat) {
				$cat = (int) trim($cat);
				if ($counter == 0) {
					$category_search .= " " . $categoryField . " = '$cat' ";
					$category_search .= " OR " . $categoryField . " = '' ";
					$category_search .= " OR " . $categoryField . " LIKE '$cat,%' ";
					$category_search .= " OR " . $categoryField . " LIKE '%,$cat,%' ";
					$category_search .= " OR " . $categoryField . " LIKE '%,$cat' ";
				} else {
					if ($cat != 0) {
						//                    $category_search .= " OR categories LIKE '%$cat%' ";
						$category_search .= " OR " . $categoryField . " = '$cat' ";
						$category_search .= " OR " . $categoryField . " LIKE '$cat,%' ";
						$category_search .= " OR " . $categoryField . " LIKE '%,$cat,%' ";
						$category_search .= " OR " . $categoryField . " LIKE '%,$cat' ";
					}
				}
				$counter++;
			}
		}
		$category_search .= ')';
		//debug($category_search);
		//debug($categories);

		// for tt_address special categorysearch
		if ($this->conf['useFeUserData'] == 4) {
			if ($categories[0] == '')
				array_shift($categories);
			for ($k = 0; $k < count($categories); $k++) {
				if ($k == 0)
					$cats = (int) $categories[$k];
				else
					$cats .= ',' . (int) $categories[$k];
			}
			$category_search = ' AND b.uid_local = a.uid AND b.uid_foreign in (' . $cats . ')';
		}


//		debug($category_search);
		//if (sizeof($categories) == 1 && $categories[0] == '') {
		// N. Badri 16.09.2011
		if (sizeof($categories) == 0 && $categories[0] == '') {
			$category_search = '';
		}

		$pi = M_PI;
		// calculating distance in km
		if ($this->conf['useFeUserData'] == 1) {
			$table = 'fe_users';
			if (t3lib_extMgm :: isLoaded('sr_feuser_register', 0))
				$fields = "*, uid as storeuid, www as url, name as storename, comments as notes, telephone as phone, zip as zipcode, tx_locator_lat as lat, tx_locator_lon as lon,(((acos(sin(($lat*$pi/180)) * sin((tx_locator_lat*$pi/180)) + cos(($lat*$pi/180)) *  cos((tx_locator_lat*$pi/180)) * cos((($lon - tx_locator_lon)*$pi/180)))))*6370) as distance";
			else // no comments field
				$fields = "*, uid as storeuid, www as url, name as storename,  telephone as phone, zip as zipcode, tx_locator_lat as lat, tx_locator_lon as lon,(((acos(sin(($lat*$pi/180)) * sin((tx_locator_lat*$pi/180)) + cos(($lat*$pi/180)) *  cos((tx_locator_lat*$pi/180)) * cos((($lon - tx_locator_lon)*$pi/180)))))*6370) as distance";

			if ($this->conf['displayMode'] == 'countryCitySelector') // search the city too
				$where_clause = 'city=' . $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['city'], 'fe_users') . ' AND ';

			$where_clause .= " pid IN (" . $this->conf['pid_list'] . ") AND (" . intval($this->conf['feUserGroup']) . " in (usergroup) OR " .
			" usergroup like '%," . intval($this->conf['feUserGroup']) . "' OR " .
			" usergroup like '%," . intval($this->conf['feUserGroup']) . ",%')" .
			" AND deleted='0' AND disable='0' $category_search HAVING distance <= $radius";
		}
		if ($this->conf['useFeUserData'] == 0) {
			$table = 'tx_locator_locations a, tx_locator_categories b';
			$fields = " distinct a.*, a.uid as storeuid, imageurl as image, (((acos(sin(($lat*$pi/180)) * sin((lat*$pi/180)) + cos(($lat*$pi/180)) *  cos((lat*$pi/180)) * cos((($lon - lon)*$pi/180)))))*6370) as distance";
			$fields .= ", (select group_concat(c.name separator ', ') from tx_locator_categories c where (a.categoryuid = c.uid OR a.categoryuid like concat('%,',c.uid) OR a.categoryuid like concat(c.uid,',%') OR a.categoryuid like concat('%,', concat(c.uid,',%')))) as name";
			$fields .= ", (select group_concat(c.name, ':', c.icon separator ', ') from tx_locator_attributes c where (a.attributes = c.uid OR a.attributes like concat('%,',c.uid) OR a.attributes like concat(c.uid,',%') OR a.attributes like concat('%,', concat(c.uid,',%')))) as attributes";
			$where_clause = " a.pid IN (" . $this->conf['pid_list'] . ")";

			$where_clause .= str_replace('tx_locator_locations', 'a', $this->cObj->enableFields('tx_locator_locations'));

			if ($this->conf['limitResultsToCountry']) {
				$where_clause .= ' AND a.country = (select cn_short_en from static_countries where cn_iso_2 = ' . $GLOBALS['TYPO3_DB']->fullQuoteStr(strtoupper($this->_GP['country']), 'tx_locator_locations') . ')';
			}


/*
debug($this->conf['categories']);
			if ($this->conf['categories']) {
			    $category_search = ' AND a.categoryuid in (' . $this->conf['categories'] . ')';;
			}
*/
			if (count($products) > 0)
				$where_clause .= $products_search;
			if ($category_search != '')
				$where_clause .= " AND b.uid = a.categoryuid"; // this works even if there are multiple categories in categoryuid

			if ($this->conf['searchByName'])
				$where_clause .= ' AND storename like ' . $GLOBALS['TYPO3_DB']->fullQuoteStr('%' . $this->_GP['storename'] . '%', 'tx_locator_locations');

			$where_clause .= " AND a.deleted='0' AND a.hidden='0' $category_search HAVING distance <= $radius";

			if ($this->conf['singleView.']['showRelatedStores']) {
				$where_clause .= ' AND relatedto=""';
			}


 			if ($this->conf['notepadId'] == $GLOBALS['TSFE']->id && $this->conf['enableNotepad'] && $_COOKIE['notepadStoreUids']) {
				if (preg_match('/^[0-9,]*$/', $_COOKIE['notepadStoreUids']) == 1)
				    $uidList = $_COOKIE['notepadStoreUids'];
				else $uidList = '';
			    $where_clause .= ' AND a.uid in (' . $uidList . ')';
 			}
		}


		if ($this->conf['useFeUserData'] == 2) {
			$table = $this->conf['externalLocationTable'];
			$fields = "*, tx_locator_categoryuid as categoryuid, link as url, name as storename, description as notes,  zip as zipcode, tx_locator_lat as lat, tx_locator_lon as lon,(((acos(sin(($lat*$pi/180)) * sin((tx_locator_lat*$pi/180)) + cos(($lat*$pi/180)) *  cos((tx_locator_lat*$pi/180)) * cos((($lon - tx_locator_lon)*$pi/180)))))*6370) as distance";
			$where_clause = " pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0' $category_search HAVING distance <= $radius";
		}


		if ($this->conf['useFeUserData'] == 3) {
// 05.09.2013
//debug($categories);
          if ($categories)
			$category_search = ' AND uid in (select uid_local from tt_news_cat_mm where uid_foreign in (' . implode(',', $categories) . '))';
			if ($this->conf['categories'])
				$category_search = ' AND uid in (select uid_local from tt_news_cat_mm where uid_foreign in (' . implode(',', $categories) . '))';

			$_categories = t3lib_div::trimExplode(',', $this->conf['categories'], 1);
			$_cat1 = array();
			$this->children = array();
			for ($i = 0; $i < count($_categories); $i++) {
				$this->children[] .= $_categories[$i];
				if ($_categories[$i] > '') {
				$_cat1 = array_merge($_cat1, $this->getChildCategories($_categories[$i]));
				$_cat1 = array_unique($_cat1);
				}
			}

            $this->conf['categories'] = $_cat1;
			if ($this->conf['categories'])
				$category_search = ' AND uid in (select uid_local from tt_news_cat_mm where uid_foreign in (' . implode(',', $_cat1) . '))';

			$table = 'tt_news a';
			$fields = "*, a.uid as storeuid, a.category as categoryuid, a.tx_locator_url as url, a.tx_locator_storename as storename, a.tx_locator_notes as notes," .
			" a.tx_locator_city as city, a.tx_locator_address as address, a.tx_locator_zipcode as zipcode," .
			" a.tx_locator_icon as icon, a.tx_locator_hours as hours, a.tx_locator_imageurl as image," .
			" a.tx_locator_phone as phone,  a.tx_locator_email as email, a.tx_locator_lat as lat, a.tx_locator_lon as lon,(((acos(sin(($lat*$pi/180)) * sin((tx_locator_lat*$pi/180)) + cos(($lat*$pi/180)) *  cos((tx_locator_lat*$pi/180)) * cos((($lon - tx_locator_lon)*$pi/180)))))*6370) as distance";
			$where_clause = " pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0' $category_search HAVING distance <= $radius";
			$where_clause .= " AND tx_locator_lat != 0 AND tx_locator_lon != 0";
			$where_clause .= " AND upper(title) like " . $GLOBALS['TYPO3_DB']->fullQuoteStr('%' . $this->_GP['storename'] . '%', 'tt_news');
			$where_clause .= $whereCat;
		}

		//martin start
		if ($this->conf['useFeUserData'] == 4) {
			if (is_array($categories)) {
				for ($i = 0; $i < count($categories); $i++) {
					if ($i == 0)
						$cats = '(' . $categories[$i];
					else
						$cats .= ',' . $categories[$i];
				}
				$cats .= ')';
			}

			$table = 'tt_address a, tt_address_group_mm b';
			$fields = "distinct a.*, a.addressgroup as categoryuid, a.www as url, a.company as storename, a.description as notes," .
			" a.city, a.address, a.zip as zipcode," .
			" a.image,";


			if (!$this->conf['useApproximation'])
				$fields .= " a.phone,  a.email, a.tx_locator_lat as lat, a.tx_locator_lon as lon,(((acos(sin(($lat*$pi/180)) * sin((tx_locator_lat*$pi/180)) + cos(($lat*$pi/180)) *  cos((tx_locator_lat*$pi/180)) * cos((($lon - tx_locator_lon)*$pi/180)))))*6370) as distance";
			else {
				// approximation
				// dx = 71.5 * (lon1 - lon2); 71.5km per degree
				// dy = 111.3 * (lat1 - lat2)
				$fields .= " a.phone,  a.email, a.tx_locator_lat as lat, a.tx_locator_lon as lon, SQRT(POW($lon-tx_locator_lon,2) * 5112 + POW($lat-tx_locator_lat,2) * 12387) as distance";
			}
			//    if ($this->_GP['categories'] > '') {
			if ($this->conf['useTtaddressWithoutGroups']) {
				// may be it is a better way to create a temporary table
				// with a select with $lat < $latmax and $lat > $latmin etc
				// create temporary table ttaddresstemp as (select ...)
				// and then use the temporary table for distance
				$latmax = $lat + $radius / 111.3;
				$latmin = $lat - $radius / 111.3;
				$lonmax = $lon + $radius / 71.5;
				$lonmin = $lon - $radius / 71.5;
                $GLOBALS['TYPO3_DB']->admin_query('DROP TABLE IF EXISTS ttaddresstemp');
				$query = "CREATE TEMPORARY TABLE ttaddresstemp AS (SELECT * from tt_address where (tx_locator_lat < $latmax and tx_locator_lat > $latmin and
					tx_locator_lon < $lonmax AND tx_locator_lon > $lonmin))";
					
                $GLOBALS['TYPO3_DB']->admin_query($query);
                
				$table = 'tt_address a';
				$table = 'ttaddresstemp a';
                $category_search = '';
			}
// here seems a bug with $category_search
//$array = array ( 1 => $category_search);
//t3lib_div::devLog('addressData', 'tx_locator', 3, $array);
//$category_search = '';


			if (!$this->conf['useTtaddressWithoutGroups']) {
			$fields .= ",(select group_concat(d.title) from tt_address_group d where (
			                    d.uid =  (select group_concat(c.uid_foreign) from tt_address_group_mm c where c.uid_local = a.uid)
			                    or (select group_concat(c.uid_foreign) from tt_address_group_mm c where c.uid_local = a.uid) like concat('%,',d.uid)
			                    or (select group_concat(c.uid_foreign) from tt_address_group_mm c where c.uid_local = a.uid) like concat('%,',d.uid,',%')
			                    or (select group_concat(c.uid_foreign) from tt_address_group_mm c where c.uid_local = a.uid) like concat(d.uid,',%')
			                    )) as category";
			}
			//    }

			$where_clause .= " a.pid IN (" . $this->conf['pid_list'] . ") AND a.deleted='0' AND a.hidden='0' $category_search HAVING distance <= $radius";
//			$where_clause .= " a.pid IN (" . $this->conf['pid_list'] . ") AND a.deleted='0' AND a.hidden='0' $category_search";

//			$where_clause .= " AND (((acos(sin(($lat*$pi/180)) * sin((tx_locator_lat*$pi/180)) + cos(($lat*$pi/180)) *  cos((tx_locator_lat*$pi/180)) * cos((($lon - tx_locator_lon)*$pi/180)))))*6370) <=  $radius";

			$where_clause .= " AND tx_locator_lat != 0 AND tx_locator_lon != 0";
		}
		//martin stop

		if ($this->_GP['mode'] == 'mapAllNoFormView' || $this->conf['displayMode'] == 'mapAllView') {
			if ($this->conf['showAreas'])
				$orderBy = "lat desc";
			else
				$orderBy = "distance ASC";
			//$orderBy = "storename ASC";
			if ($this->conf['mapAllNoFormView.']['rand']) {
			    $orderBy = ' rand(now())';
				$results_limit = intval($this->conf['mapAllNoFormView.']['rand']);
			}

		} else
			$orderBy = "distance ASC";

		if ($this->conf['useFeUserData'] == 1) {
			$orderBy = 'tx_locator_toplocation desc, ' . $orderBy;
		}

		// sanitizing when called from eID-script
		if (preg_match('/^[A-z,]*$/', $this->conf['orderBy']) == 1) {
		    // all is ok
		} else $this->conf['orderBy'] = '';

		if ($this->conf['orderBy'])
		    $orderBy = $this->conf['orderBy'];


		$groupBy = "";
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where_clause, $groupBy, $orderBy, $results_limit);
		$i = 0;
		return $result;
	} // end func

	function getCenterOfAllStores() {
		// sanitizing when called from eID-script
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		$pi = M_PI;
		$this->cObj = t3lib_div :: makeInstance("tslib_cObj");
		// calculating distance in km
		if ($this->conf['useFeUserData'] == 1) {
			$table = 'fe_users';
			if (t3lib_extMgm :: isLoaded('sr_feuser_register', 0))
				$fields = "*, www as url, name as storename, comments as notes, telephone as phone, zip as zipcode, tx_locator_lat as lat, tx_locator_lon as lon";
			else // no comments field
				$fields = "*, www as url, name as storename,  telephone as phone, zip as zipcode, tx_locator_lat as lat, tx_locator_lon as lon";

			//                $where_clause = " pid IN (".$this->conf['pid_list'].") AND ". $this->conf['feUserGroup']." in (usergroup) AND deleted='0' AND disable='0' $category_search HAVING distance <= $radius";
			$where_clause = " pid IN (" . $this->conf['pid_list'] . ") AND (" . intval($this->conf['feUserGroup']) . " in (usergroup) OR " .
			" usergroup like '%," . intval($this->conf['feUserGroup']) . "' OR " .
			" usergroup like '%," . intval($this->conf['feUserGroup']) . ",%')" .
			" AND deleted='0' AND disable='0'";
		}
		if ($this->conf['useFeUserData'] == 0) {
			$table = 'tx_locator_locations';
			$fields = "*,imageurl as image";
			$where_clause = " pid IN (" . $this->conf['pid_list'] . ")";
			$where_clause .= $this->cObj->enableFields('tx_locator_locations');

// may be?
            if ($this->_GP['categories'] > 0) {
/*
				if ($this->getChildCategories($this->_GP['categories']))
				$where_clause .= ' AND categoryuid in (' . $this->_GP['categories'] . ',' . implode(',',$this->getChildCategories($this->_GP['categories'])) . ')';
				else
				$where_clause .= ' AND categoryuid =' . $this->_GP['categories'];
*/
			// sanitizing
			if (preg_match('/^[0-9,]*$/', $this->_GP['categories']) == 1) {
			    // all is ok
			} else $this->_GP['categories'] = '';


			if ($this->getChildCategories($this->_GP['categories']))
		 		$categories = $this->_GP['categories'] . ',' . implode(',', $this->getChildCategories($this->_GP['categories']));
			else $categories = $this->_GP['categories'];

			$categories = explode(',', $categories);
            $categoryField = 'categoryuid';
			foreach ($categories as $cat) {
				$cat = (int) trim($cat);
				if ($counter == 0) {
					$category_search .= " " . $categoryField . " = '$cat' ";
					$category_search .= " OR " . $categoryField . " = '' ";
					$category_search .= " OR " . $categoryField . " LIKE '$cat,%' ";
					$category_search .= " OR " . $categoryField . " LIKE '%,$cat,%' ";
					$category_search .= " OR " . $categoryField . " LIKE '%,$cat' ";
				} else {
					if ($cat != 0) {
						//                    $category_search .= " OR categories LIKE '%$cat%' ";
						$category_search .= " OR " . $categoryField . " = '$cat' ";
						$category_search .= " OR " . $categoryField . " LIKE '$cat,%' ";
						$category_search .= " OR " . $categoryField . " LIKE '%,$cat,%' ";
						$category_search .= " OR " . $categoryField . " LIKE '%,$cat' ";
					}
				}
				$counter++;
			}
            $where_clause .= ' AND ' . $category_search;

			}
			
			// notepad stores
			if ($this->conf['pageId'] == $this->conf['notepadId'] && $this->conf['enableNotepad'] && $_COOKIE['notepadStoreUids']) {
//debug($_COOKIE['notepadStoreUids'], 'getCenterOfAllStores');
				if (preg_match('/^[0-9,]*$/', $_COOKIE['notepadStoreUids']) == 1)
				    $uidList = $_COOKIE['notepadStoreUids'];
				else $uidList = '';

			    $where_clause .= ' AND uid in (' . $uidList . ')';
			}

		}

		if ($this->conf['useFeUserData'] == 2) {
			$table = $this->conf['externalLocationTable'];
			$fields = "*, tx_locator_categoryuid as categoryuid, link as url, name as storename, description as notes,  zip as zipcode, tx_locator_lat as lat, tx_locator_lon as lon";
			$where_clause = " pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
		}

		if ($this->conf['useFeUserData'] == 3) {
			$table = 'tt_news';
			$where_clause = " tx_locator_lat != 0 AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
//			$where_clause .= ' AND category =' . $this->conf['categorySelect'];
			$fields = "*, tx_locator_zipcode as zipcode, tx_locator_address as address, tx_locator_city as city," .
			" tx_locator_phone as phone, tx_locator_url as url, tx_locator_notes as notes, title as storename, image as imageurl, tx_locator_lat as lat, tx_locator_lon as lon";
			$orderBy = '';
		}
		// martin start
		if ($this->conf['useFeUserData'] == 4) {
			$table = 'tt_address';
			$where_clause = " tx_locator_lat != 0 AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
			$fields = "*, zip as zipcode, address, city," .
			" phone, www as url, description as notes, company as storename, image as imageurl, tx_locator_lat as lat, tx_locator_lon as lon";
			$orderBy = '';
		}
		//martin stop

		$orderBy = "";
		$groupBy = "";
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where_clause, $groupBy, $orderBy);
		$i = 1;
		while ($rows = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			if ($i) {
				$i = 0;
				$minLat = $rows['lat'];
				$minLon = $rows['lon'];
			}
			$maxLat = max($maxLat, $rows['lat']);
			$minLat = min($minLat, $rows['lat']);
			$maxLon = max($maxLon, $rows['lon']);
			$minLon = min($minLon, $rows['lon']);
		}
		
		$center = new \stdclass();
		$center->lat = ($maxLat + $minLat) / 2;
		$center->lon = ($maxLon + $minLon) / 2;
		$center->radius = number_format(max($maxLat - $minLat, $maxLon - $minLon) / 360 * 40000, 2, '.', '');
		return $center;
	}

	function getCenter() {
		$minLat = ((float)$this->_GP['lat'] > '') ? (float)$this->_GP['lat'] : 90;
		$minLon = ((float)$this->_GP['lon'] > '') ? (float)$this->_GP['lon'] : 180;
		for ($i = 0; $i < count($this->_GP['relatedTo']['lat']); $i++) {
			$maxLat = max($maxLat, $this->_GP['relatedTo']['lat'][$i]);
			$minLat = min($minLat, $this->_GP['relatedTo']['lat'][$i]);
			$maxLon = max($maxLon, $this->_GP['relatedTo']['lon'][$i]);
			$minLon = min($minLon, $this->_GP['relatedTo']['lon'][$i]);
		}
		$center = new \stdclass();
		$center->lat = ($maxLat + $minLat) / 2;
		$center->lon = ($maxLon + $minLon) / 2;
		$center->radius = number_format(max($maxLat - $minLat, $maxLon - $minLon) / 360 * 40000, 2, '.', '');
		if ($center->radius = '0.00')
			$center->radius = '3.00';
		return $center;
	}

	function getFirstStore() {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';
		// when storeUid is supplied take this record
		if ($this->_GP['storeUid'])
			$this->conf['uidOfSingleView'] = intval($this->_GP['storeUid']);

		if ($this->conf['useFeUserData'] == 1) {
			$table = 'fe_users';
			//$where_clause = " uid = ".$this->conf['uidOfSingleView'] . " AND pid IN (".$this->conf['pid_list'].") AND ". $this->conf['feUserGroup']." in (usergroup) AND deleted='0' AND disable='0'";
			$where_clause = " uid = " . intval($this->conf['uidOfSingleView']) . " AND pid IN (" . $this->conf['pid_list'] . ") AND (" . intval($this->conf['feUserGroup']) . " in (usergroup) OR (usergroup) LIKE '%," . intval($this->conf['feUserGroup']) . "' OR (usergroup) LIKE '%," . intval($this->conf['feUserGroup']) . ",%' ) AND deleted='0' AND disable='0'";
			$fields = "*, www as url, zip as zipcode, telephone as phone, name as storename, image as imageurl, comments as notes, tx_locator_lat as lat, tx_locator_lon as lon";
			$orderBy = '';
		}
		if ($this->conf['useFeUserData'] == 0) {
			$table = 'tx_locator_locations a';
			$where_clause = " a.uid ='" . $this->conf['uidOfSingleView'] . "' AND a.pid IN (" . $this->conf['pid_list'] . ") AND a.deleted='0' AND a.hidden='0'";
			$fields = '*';
			$fields .= ", (select group_concat(c.name) from tx_locator_categories c where (a.categoryuid = c.uid OR a.categoryuid like concat('%,',c.uid) OR a.categoryuid like concat(c.uid,',%') OR a.categoryuid like concat('%,', concat(c.uid,',%')))) as name";
			$fields .= ", (select group_concat(c.name, ':', c.icon separator ', ') from tx_locator_attributes c where (a.attributes = c.uid OR a.attributes like concat('%,',c.uid) OR a.attributes like concat(c.uid,',%') OR a.attributes like concat('%,', concat(c.uid,',%')))) as attributes";
			$orderBy = 'sorting';
		}
		if ($this->conf['useFeUserData'] == 2) {
			$table = $this->conf['externalLocationTable'];
			$where_clause = " uid ='" . intval($this->conf['uidOfSingleView']) . "' AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
			$fields = "*, zip as zipcode, street as address, link as url, description as notes, name as storename, image as imageurl, tx_locator_lat as lat, tx_locator_lon as lon";
			if ($this->conf['externalLocationTable'] == 'tx_flatmgr_flat')
				$fields = "*, name as storename, zip as zipcode, street as address, description as notes, pic as imageurl, tx_locator_lat as lat, tx_locator_lon as lon";
			$orderBy = '';
		}
		if ($this->conf['useFeUserData'] == 3) {
			$table = 'tt_news';
			$where_clause = " uid ='" . intval($this->conf['uidOfSingleView']) . "' AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
			$fields = "*, tx_locator_zipcode as zipcode, tx_locator_address as address, tx_locator_city as city,tx_locator_icon as icon," .
			" tx_locator_phone as phone, tx_locator_url as url, tx_locator_notes as notes, tx_locator_storename as storename, image as imageurl, tx_locator_lat as lat, tx_locator_lon as lon";
			$orderBy = '';
		}
		if ($this->conf['useFeUserData'] == 4) {
			$table = 'tt_address';
			$where_clause = " uid ='" . intval($this->conf['uidOfSingleView']) . "' AND pid IN (" . $this->conf['pid_list'] . ") AND deleted='0' AND hidden='0'";
			$fields = "*, zip as zipcode, address, city," .
			" phone, www as url, description as notes, " . $this->conf['tt_addressStorenameField'] . " as storename, image as imageurl, tx_locator_lat as lat, tx_locator_lon as lon";
			$orderBy = '';
		}

		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where_clause, '', $orderBy);
		$out = array ();
		$row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result);

		$this->_GP['uid'] = htmlspecialchars($row['uid']);
		$this->_GP['lat'] = htmlspecialchars($row['lat']);
		$this->_GP['lon'] = htmlspecialchars($row['lon']);
		$this->_GP['storename'] = htmlspecialchars($row['storename'], ENT_QUOTES);
		$this->_GP['nameTitle'] = htmlspecialchars($row['title']);
		$this->_GP['firstName'] = htmlspecialchars($row['first_name']);
		$this->_GP['lastName'] = htmlspecialchars($row['last_name']);
		$this->_GP['storeId'] = htmlspecialchars($row['storeid']);
		$this->_GP['attributes'] = htmlspecialchars($row['attributes']);
		$this->_GP['category'] = htmlspecialchars(str_replace(',', ', ', $row['name']));
		$this->_GP['contactPerson'] = htmlspecialchars($row['contactperson']);
		$this->_GP['company'] = htmlspecialchars($row['company']);
		$this->_GP['zipcode'] = htmlspecialchars($row['zipcode']);
		$this->_GP['city'] = htmlspecialchars($row['city']);
		$this->_GP['address'] = htmlspecialchars($row['address']);
		$this->_GP['additionalAddress'] = htmlspecialchars($row['additionaladdress']);
		$this->_GP['country'] = htmlspecialchars($row['country']);
		$this->_GP['icon'] = htmlspecialchars($row['icon']);
		$this->_GP['phone'] = htmlspecialchars($row['phone']);
		$this->_GP['mobile'] = htmlspecialchars($row['mobile']);
		$this->_GP['fax'] = htmlspecialchars($row['fax']);
		$this->_GP['email'] = htmlspecialchars($row['email']);
		$this->_GP['hours'] = htmlspecialchars($row['hours']);
		$this->_GP['notes'] = 	$row['notes'];
		$this->_GP['link'] = htmlspecialchars($row['url']);
		$this->_GP['image'] = htmlspecialchars($row['imageurl']);
		$this->_GP['content'] = htmlspecialchars($row['content']);
		$this->_GP['media'] = htmlspecialchars($row['media']);

		if (t3lib_extMgm :: isLoaded('locatorfeuser')) {
			require_once (t3lib_extMgm :: extPath('locatorfeuser') . "pi1/class.tx_locatorfeuser_model.php");

			$model = new tx_locatorfeuser_model();
			$nameTitle = $model->getFeUserTitle($row['uid']);
			$this->_GP['tx_locatorfeuser_nameTitle'] = htmlspecialchars($nameTitle);
			$this->_GP['nameTitle'] = htmlspecialchars($nameTitle);
			$this->_GP['tx_locatorfeuser_firstname'] = htmlspecialchars($row['tx_locatorfeuser_firstname']);
			$this->_GP['locatorfeuser_qualification'] = htmlspecialchars($row['tx_locatorfeuser_qualification']);
			$this->_GP['locatorfeuser_generaltraining'] = htmlspecialchars($row['tx_locatorfeuser_generaltraining']);
			$this->_GP['locatorfeuser_actualtraining'] = htmlspecialchars($row['tx_locatorfeuser_actualtraining']);
			$this->_GP['locatorfeuser_references'] = htmlspecialchars($row['tx_locatorfeuser_references']);
			$this->_GP['locatorfeuser_prefbilling'] = htmlspecialchars($row['tx_locatorfeuser_prefbilling']);
		}
		if (t3lib_extMgm :: isLoaded('feuser_open')) {
			$moVo = htmlspecialchars($row['tx_feuseropen_montag_vormittag']);
			$diVo = htmlspecialchars($row['tx_feuseropen_dienstag_vormittag']);
			$miVo = htmlspecialchars($row['tx_feuseropen_mittwoch_vormittag']);
			$doVo = htmlspecialchars($row['tx_feuseropen_donnerstag_vormittag']);
			$frVo = htmlspecialchars($row['tx_feuseropen_freitag_vormittag']);

			$moNa = htmlspecialchars($row['tx_feuseropen_montag_nachmittag']);
			$diNa = htmlspecialchars($row['tx_feuseropen_dienstag_nachmittag']);
			$miNa = htmlspecialchars($row['tx_feuseropen_mittwoch_nachmittag']);
			$doNa = htmlspecialchars($row['tx_feuseropen_donnerstag_nachmittag']);
			$frNa = htmlspecialchars($row['tx_feuseropen_freitag_nachmittag']);

			$this->_GP['tx_feuseropen_montag_vormittag'] = $moVo;
			$this->_GP['tx_feuseropen_dienstag_vormittag'] = $diVo;
			$this->_GP['tx_feuseropen_mittwoch_vormittag'] = $miVo;
			$this->_GP['tx_feuseropen_donnerstag_vormittag'] = $doVo;
			$this->_GP['tx_feuseropen_freitag_vormittag'] = $frVo;

			$this->_GP['tx_feuseropen_montag_nachmittag'] = $moNa;
			$this->_GP['tx_feuseropen_dienstag_nachmittag'] = $diNa;
			$this->_GP['tx_feuseropen_mittwoch_nachmittag'] = $miNa;
			$this->_GP['tx_feuseropen_donnerstag_nachmittag'] = $doNa;
			$this->_GP['tx_feuseropen_freitag_nachmittag'] = $frNa;
		}

		$this->_GP['distance'] = (float)$this->_GP['distance'] .
		' ' . $this->pi_getLL($this->conf['distanceUnit']);
		//debug($this->_GP, 'model');
		return $this->_GP;
	}

	function getRelatedStores() {
		$GLOBALS['TYPO3_DB']->debugOutput = 1;
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';
		
		$this->cObj = t3lib_div :: makeInstance("tslib_cObj");
		$table = 'tx_locator_locations a';
		$fields = '*';
		$where = " relatedto=" . intval($this->_GP['uid']) . " and a.pid IN (" . $this->conf['pid_list'] . ")";
		$where .= str_replace('tx_locator_locations', 'a', $this->cObj->enableFields('tx_locator_locations'));
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, $groupBy, $orderBy, $results_limit);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$data['uid'][] .= htmlspecialchars($row['uid']);
			$data['storename'][] .= htmlspecialchars($row['storename']);
			//            $data['category'][]            .= str_replace(',', ', ', $row['name']);
			$data['address'][] .= htmlspecialchars($row['address']);
			$data['additionalAddress'][] .= htmlspecialchars($row['additionaladdress']);
			$data['zipcode'][] .= htmlspecialchars($row['zipcode']);
			$data['city'][] .= htmlspecialchars($row['city']);
			$data['contactPerson'][] .= htmlspecialchars($row['contactperson']);
			$data['country'][] .= htmlspecialchars($row['country']);
			$data['email'][] .= htmlspecialchars($row['email']);
			$data['phone'][] .= htmlspecialchars($row['phone']);
			$data['fax'][] .= htmlspecialchars($row['fax']);
			$data['mobile'][] .= htmlspecialchars($row['mobile']);
			$data['url'][] .= htmlspecialchars($row['url']);
			$data['hours'][] .= htmlspecialchars($row['hours']);
			$data['notes'][] .= htmlspecialchars($row['notes']);
			$data['image'][] .= htmlspecialchars($row['imageurl']);
			$data['icon'][] .= htmlspecialchars($row['icon']);
			$data['attributes'][] .= htmlspecialchars($row['attributes']);
			$data['lat'][] .= htmlspecialchars($row['lat']);
			$data['lon'][] .= htmlspecialchars($row['lon']);
		}
		return $data;
	}

	function getAllStores() {
		$this->cObj = t3lib_div :: makeInstance("tslib_cObj");
		$table = 'tx_locator_locations a';
		$fields = " distinct *,imageurl as image,(select group_concat(c.name) from tx_locator_categories c where (a.categoryuid = c.uid OR a.categoryuid like concat('%,',c.uid) OR a.categoryuid like concat(c.uid,',%') OR a.categoryuid like concat('%,', concat(c.uid,',%')))) as name";


		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['listViewCategoryUid']) == 1) {
		    // all is ok
		} else $this->conf['listViewCategoryUid'] = '';

		// for tt_news
		if ($this->conf['useFeUserData'] == 3) {
				if ($this->conf['listViewCategoryUid']) {
					$table = 'tt_news a, tt_news_cat_mm b';
					$where_clause = 'a.uid = b.uid_local and b.uid_foreign in ( ' . $this->conf['listViewCategoryUid'] . ') AND ';
				}
				else $table = 'tt_news a';

				$fields = " distinct *, tx_locator_storename as storename, tx_locator_zipcode as zipcode, tx_locator_city as city, tx_locator_address as address,
					tx_locator_country as country, short as contactperson, tx_locator_hours as hours, tx_locator_url as url,
					tx_locator_email as email, tx_locator_phone as phone";


		}

		$where_clause .= " a.pid IN (" . $this->conf['pid_list'] . ")";
		$where_clause .= str_replace('tx_locator_locations', 'a', $this->cObj->enableFields('tx_locator_locations'));
		if ($this->_GP['categoryUid'] > 0) {
			//            $where_clause .= ' AND ' . intval($this->_GP['categoryUid']) . ' in (a.categoryuid)';
			$where_clause .= ' AND ( a.categoryuid =' . intval($this->_GP['categoryUid']) . ' OR a.categoryuid like concat("%,",' . intval($this->_GP['categoryUid']) . ') OR a.categoryuid like concat("%,",' . intval($this->_GP['categoryUid']) . ',",%") OR a.categoryuid like concat(' . intval($this->_GP['categoryUid']) . ',",%") )';

		}

		if ($this->conf['listViewCategoryUid'] && $this->conf['useFeUserData'] != 3) {
			// sanitizing
			if (preg_match('/^[0-9,]*$/', $this->conf['listViewCategoryUid']) == 1) {
			    // all is ok
			} else $this->conf['listViewCategoryUid'] = '';
			$where_clause .= ' AND a.categoryuid in (' . $this->conf['listViewCategoryUid'] . ')';
		}

 		if ($this->_GP['storename'] > '')
			$where_clause .= ' AND upper(storename) like concat("%",' . $GLOBALS['TYPO3_DB']->fullQuoteStr(strtoupper($this->_GP['storename']), $table) . ',"%")';
 		if ($this->_GP['kitchen'] > '')
			$where_clause .= ' AND upper(tx_locatorparking_kitchen) like concat("%",' . $GLOBALS['TYPO3_DB']->fullQuoteStr(strtoupper($this->_GP['kitchen']), $table) . ',"%")';
 		if ($this->_GP['trademark'] > '')
			$where_clause .= ' AND (upper(tx_locatorparking_trademark) like concat("%",' . $GLOBALS['TYPO3_DB']->fullQuoteStr(strtoupper($this->_GP['trademark']), $table) . ',"%") OR' .
			' upper(tx_locatorparking_assortment) like concat("%",' . $GLOBALS['TYPO3_DB']->fullQuoteStr(strtoupper($this->_GP['trademark']), $table) . ',"%"))';
 		if ($this->_GP['branch'] > '')
			$where_clause .= ' AND upper(tx_locatorparking_branch) like concat("%",' . $GLOBALS['TYPO3_DB']->fullQuoteStr(strtoupper($this->_GP['branch']), $table) . ',"%")';

		if ($this->conf['singleView.']['showRelatedStores']) {
			$where_clause .= ' AND relatedto=""';
		}

		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->_GP['uidsToHideInOtherMatches']) == 1) {
		    // all is ok
		} else $this->_GP['uidsToHideInOtherMatches'] = '';

		if ($this->_GP['uidsToHideInOtherMatches'])
			$where_clause .= ' AND uid not in (' . $this->_GP['uidsToHideInOtherMatches'] . ')';
		if ($this->conf['listViewAddWhere'])
			$where_clause .= ' AND ' . $this->conf['listViewAddWhere'];
		if (!$this->conf['listViewOrderBy'])
			$orderBy = 'country, zipcode';
		else
			$orderBy = $this->conf['listViewOrderBy'];

		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where_clause, $groupBy, $orderBy, $results_limit);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			if ($this->_GP['exactSearch']) {
				$_trademarks = t3lib_div::trimExplode(',', htmlspecialchars($row['tx_locatorparking_trademark']), 1);
				$found = 0;
				for ($i = 0; $i < count($_trademarks); $i++) {
				    if (strtoupper($_trademarks[$i]) == $GLOBALS['TYPO3_DB']->fullQuoteStr(strtoupper(htmlspecialchars($this->_GP['trademark'])), $table)) {
						$found = 1;
					}
				}
				$_assortments = t3lib_div::trimExplode(',', htmlspecialchars($row['tx_locatorparking_assortment']), 1);

				for ($i = 0; $i < count($_assortments); $i++) {
				    if (strtoupper($_assortments[$i]) == $GLOBALS['TYPO3_DB']->fullQuoteStr(strtoupper(htmlspecialchars($this->_GP['trademark'])), $table)) {
						$found = 1;
					}
				}
			}
			if ( ($this->_GP['exactSearch'] && $found) || !$this->_GP['exactSearch']) {
				$data['uid'][] .= htmlspecialchars($row['uid']);
				$data['storeid'][] .= htmlspecialchars($row['storeid']);
				$data['storename'][] .= htmlspecialchars($row['storename']);
				$data['category'][] .= htmlspecialchars(str_replace(',', ', ', $row['name']));
				$data['address'][] .= htmlspecialchars($row['address']);
				$data['additionalAddress'][] .= htmlspecialchars($row['additionaladdress']);
				$data['zipcode'][] .= htmlspecialchars($row['zipcode']);
				$data['city'][] .= htmlspecialchars($row['city']);
				$data['contactPerson'][] .= htmlspecialchars($row['contactperson']);
				$data['country'][] .= htmlspecialchars($row['country']);
				$data['email'][] .= htmlspecialchars($row['email']);
				$data['phone'][] .= htmlspecialchars($row['phone']);
				$data['fax'][] .= htmlspecialchars($row['fax']);
				$data['mobile'][] .= htmlspecialchars($row['mobile']);
				$data['nameTitle'][] .= htmlspecialchars($row['title']);
				$data['url'][] .= htmlspecialchars($row['url']);
				$data['hours'][] .= nl2br(htmlspecialchars($row['hours']));
//				$data['notes'][] .= htmlspecialchars($row['notes']);
				$data['notes'][] .= $row['notes'];
				$data['image'][] .= htmlspecialchars($row['image']);
				$data['attributes'][] .= htmlspecialchars($row['attributes']);
				$data['lat'][] .= htmlspecialchars($row['lat']);
				$data['lon'][] .= htmlspecialchars($row['lon']);
				$data['area'][] .= htmlspecialchars($row['area']);
			}

		}
		return $data;
	}

	function getCountries() {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		if ($this->conf['useFeUserData'] == 0)
			$table = 'tx_locator_locations a';
		if ($this->conf['useFeUserData'] == 1)
			$table = 'fe_users a';

		$fields = " distinct country";
		$where = " a.pid IN (" . $this->conf['pid_list'] . ")";
		if ($this->conf['useFeUserData'] == 0)
			$where .= " AND a.deleted='0' AND a.hidden='0'";
		if ($this->conf['useFeUserData'] == 1)
			$where .= " AND a.deleted='0' AND a.disable='0'";
		$orderBy = 'country';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, $groupBy, $orderBy, $results_limit);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$data['country'][] .= htmlspecialchars($row['country']);
		}
		return $data;
	}

	function getCnIso2($country) {
		$table = 'static_countries a';
		$fields = "cn_iso_2";
		$where = " cn_short_de=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($country, 'static_countries');
		$where .= " AND a.deleted='0'";
		$orderBy = '';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, $groupBy, $orderBy, $results_limit);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$cn_iso_2 = $row['cn_iso_2'];
		}
		return $cn_iso_2;
	}

	function getCities() {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		if ($this->conf['useFeUserData'] == 0)
			$table = 'tx_locator_locations a';
		if ($this->conf['useFeUserData'] == 1)
			$table = 'fe_users a';
		$fields = " distinct city";
		$where = " a.pid IN (" . $this->conf['pid_list'] . ")";
		if ($this->conf['useFeUserData'] == 0)
			$where .= " AND country=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['country'], 'tx_locator_locations') . " AND a.deleted='0' AND a.hidden='0'";

		if ($this->conf['useFeUserData'] == 1)
			$where .= " AND country=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['country'], 'fe_users') . " AND a.deleted='0' AND a.disable='0'";
		$orderBy = 'city';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, $groupBy, $orderBy, $results_limit);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$data['city'][] .= $row['city'];
			$count = $this->getStoresCount($row['city']);
			if ($count == 1) {
				$storeUid = $this->getStoreUidOfCity($row['city']); // needed for realURL title
			} else
				$storeUid = 0;
			$data['count'][] .= $count;
			$data['storeUid'][] .= $storeUid;
		}
		return $data;
	}

	function getStoresCount($city) {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		$this->cObj = t3lib_div :: makeInstance('tslib_cObj');
		$where .= " a.pid IN (" . $this->conf['pid_list'] . ")";
		if ($this->conf['useFeUserData'] == 0) {
			$table = 'tx_locator_locations a';
			$where .= str_replace('tx_locator_locations', 'a', $this->cObj->enableFields('tx_locator_locations'));
		}
		if ($this->conf['useFeUserData'] == 1) {
			$table = 'fe_users a';
			$where .= str_replace('fe_users', 'a', $this->cObj->enableFields('fe_users'));
		}
		$fields = " count(*) as count";
		$where .= " AND a.pid IN (" . $this->conf['pid_list'] . ")";

		if ($this->conf['useFeUserData'] == 0)
			$where .= " AND city=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($city, 'tx_locator_locations') . " AND country=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['country'], 'tx_locator_locations') . " AND a.deleted='0' AND a.hidden='0'";

		if ($this->conf['useFeUserData'] == 1)
			$where .= " AND city=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($city, 'fe_users') . " AND country=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['country'], 'fe_users') . " AND a.deleted='0' AND a.disable='0'";

		$orderBy = 'city';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, $groupBy, $orderBy, $results_limit);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$count = $row['count'];
		}
		return $count;
	}

	function getStoreUidOfCity($city) {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		if ($this->conf['useFeUserData'] == 0)
			$table = 'tx_locator_locations a';
		if ($this->conf['useFeUserData'] == 1)
			$table = 'fe_users a';
		$fields = " uid";
		$where = " a.pid IN (" . $this->conf['pid_list'] . ")";
		if ($this->conf['useFeUserData'] == 0)
			$where .= " AND city=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($city, 'tx_locator_locations') . " AND country=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['country'], 'tx_locator_locations') . " AND a.deleted='0' AND a.hidden='0'";

		if ($this->conf['useFeUserData'] == 1)
			$where .= " AND city=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($city, 'fe_users') . " AND country=" . $GLOBALS['TYPO3_DB']->fullQuoteStr($this->_GP['country'], 'fe_users') . " AND a.deleted='0' AND a.disable='0'";
		$orderBy = 'city';
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, $groupBy, $orderBy, $results_limit);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$uid = $row['uid'];
		}
		return $uid;
	}

	function searchAllStores() {
		$start = array ();
		$this->_GP['start'] = array ();
		if ($this->_GP['lat']) {
			$center_lat_lon->lat = (float) $this->_GP['lat'];
			$center_lat_lon->lon = (float) $this->_GP['lon'];
		} else
			$center_lat_lon = $this->geocodeAddress($this->_GP);

		if (count($this->_GP['start']['city']) > 1) {
			// more than one city found for startingpoint
			// show city selection
			$subpart = $this->cObj->getSubpart($this->template, "###SEARCHCITY_HEADER###");
			$marksHeader['###MULTISTARTINGPOINTS###'] = $this->pi_getLL('multiStartingPoints');

			$final = $this->cObj->substituteMarkerArray($subpart, $marksHeader);

			$subpart = $this->cObj->getSubpart($this->template, "###SEARCHCITY_DATA###");
			$lon = (float) $this->_GP['start']['lon'];
			$lat = (float) $this->_GP['start']['lat'];
			$i = 0;

			while (list ($key, $v) = each($this->_GP['start']['city'])) {
				//$marks['###FORMHEADER###'] = $this->pi_getLL('formHeader');
				$marks['###FORMSTART###'] = '<form method="post" action="' . $this->pi_getPageLink($GLOBALS['TSFE']->id) . '">';
				$marks['###CITYV###'] = htmlspecialchars($key);
				$marks['###RADIUSV###'] = (int) $this->_GP['radius'];
				$marks['###COUNTRYV###'] = htmlspecialchars($this->_GP['country']);
				$marks['###LATV###'] = (float) $lat[$i];
				$marks['###LONV###'] = (float) $lon[$i];
				$marks['###STORESEARCH###'] = $this->pi_getLL('storeSearch');

				$i++;
				$final .= $this->cObj->substituteMarkerArray($subpart, $marks);

			}
			$final .= $this->cObj->getSubpart($this->template, "###SEARCHCITY_FOOTER###");
			return $final;
		}

		$HTTP_POST_VARS['lat_lon'] = $center_lat_lon;

		$this->_GP['lat_lon'] = $center_lat_lon;
		/*
		 * get the stores
		 */
		$mysqlResults = $this->inradius($this->_GP, (int) $this->_GP['radius']); // int for security
		return $mysqlResults;
	}

	function searchStore() {
		$this->cObj = t3lib_div :: makeInstance("tslib_cObj");
		$this->template = $this->cObj->fileResource($this->conf["templateFile"]);
		$start = array ();
		$this->_GP['start'] = array ();
		$center_lat_lon = new \stdclass();

		if ($this->_GP['lat']) {
			$center_lat_lon->lat = (float) $this->_GP['lat'];
			$center_lat_lon->lon = (float) $this->_GP['lon'];
		} else
			$center_lat_lon = $this->geocodeAddress($this->_GP);

		if (count($this->_GP['start']['city']) > 1) {
			// more than one city found for startingpoint
			// show city selection
			$subpart = $this->cObj->getSubpart($this->template, "###SEARCHCITY_HEADER###");
			$marksHeader['###MULTISTARTINGPOINTS###'] = $this->pi_getLL('multiStartingPoints');

			$final = $this->cObj->substituteMarkerArray($subpart, $marksHeader);

			$subpart = $this->cObj->getSubpart($this->template, "###SEARCHCITY_DATA###");
			$lon = (float) $this->_GP['start']['lon'];
			$lat = (float) $this->_GP['start']['lat'];
			$i = 0;

			while (list ($key, $v) = each($this->_GP['start']['city'])) {
				//$marks['###FORMHEADER###'] = $this->pi_getLL('formHeader');
				$marks['###FORMSTART###'] = '<form method="post" action="' . $this->pi_getPageLink($GLOBALS['TSFE']->id) . '">';
				$marks['###CITYV###'] = ($GLOBALS['TSFE']->localeCharset == 'utf-8') ? htmlspecialchars(utf8_encode($key)) : htmlspecialchars($key);
				$marks['###RADIUSV###'] = (int) $this->_GP['radius'];
				$marks['###COUNTRYV###'] = $this->_GP['country'];
				$marks['###LATV###'] = (float) $lat[$i];
				$marks['###LONV###'] = (float) $lon[$i];
				$marks['###STORESEARCH###'] = $this->pi_getLL('storeSearch');
				$i++;
				$final .= $this->cObj->substituteMarkerArray($subpart, $marks);

			}
			$final .= $this->cObj->getSubpart($this->template, "###SEARCHCITY_FOOTER###");
			return $final;
		}

		$HTTP_POST_VARS['lat_lon'] = $center_lat_lon;
		$this->_GP['lat_lon'] = $center_lat_lon;
		/*
		 * get the stores
		 */
		$mysqlResults = $this->inradius($this->_GP, (int) $this->_GP['radius']);
		if ($mysqlResults == NULL) {
			return '<div class="searchResultHeader">' . $this->pi_getLL('noStoresFound') . '</div>';
		}
		$data['mysql'] = $mysqlResults;
		$data['center'] = $center_lat_lon;

		return $data;
	}

	function testZipcode($zipcode) {
		// sanitizing
		if (preg_match('/^[A-z0-9 -]*$/', $zipcode) == 1) {
		    // all is ok
		} else $zipcode = '';
		$table = 'tx_locator_zipcodeareas';
		$fields = 'count(*) as counts';
		$where = 'zipcode=' . $zipcode;
		$orderBy = 'uid';
		$groupBy = '';

		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, $groupBy, $orderBy);
		$row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result);
		$counts .= $row['counts'];
		return $counts;
	}

	function searchZipcodeAreas() {
		// sanitizing
		if (preg_match('/^[0-9,]*$/', $this->conf['pid_list']) == 1) {
		    // all is ok
		} else $this->conf['pid_list'] = '';

		$table = 'tx_locator_zipcodeareas a, tx_locator_locations b';
		$fields = 'a.uid, a.zipcode, a.lat, a.lon, a.area';
		$where = ' a.zipcode = b.zipcode AND b.hidden = 0 AND b.deleted = 0';
		$where .= ' AND a.pid IN (' . $this->conf['pid_list'] . ')';
		$orderBy = 'a.uid';
		$groupBy = '';

		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $table, $where, $groupBy, $orderBy);
		while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
			$data['uid'][] .= htmlspecialchars($row['uid']);
			$data['zipcode'][] .= htmlspecialchars($row['zipcode']);
			$data['lat'][] .= htmlspecialchars($row['lat']);
			$data['lon'][] .= htmlspecialchars($row['lon']);
			$data['area'][] .= htmlspecialchars($row['area']);

		}
		return $data;
	}

	function listMailStores() {
		$start = array ();
		$this->_GP['start'] = array ();
		if ($this->_GP['lat']) {
			$center_lat_lon->lat = (float) $this->_GP['lat'];
			$center_lat_lon->lon = (float) $this->_GP['lon'];
		} else
			$center_lat_lon = $this->geocodeAddress($this->_GP);

		if (count($this->_GP['start']['city']) > 1) {
			// more than one city found for startingpoint
			// show city selection
			$subpart = $this->cObj->getSubpart($this->template, "###SEARCHCITY_HEADER###");
			$marksHeader['###MULTISTARTINGPOINTS###'] = $this->pi_getLL('multiStartingPoints');

			$final = $this->cObj->substituteMarkerArray($subpart, $marksHeader);

			$subpart = $this->cObj->getSubpart($this->template, "###SEARCHCITY_DATA###");
			$lon = array_keys($this->_GP['start']['lon']);
			$lat = array_keys($this->_GP['start']['lat']);
			$i = 0;
			while (list ($key, $v) = each($this->_GP['start']['city'])) {
				//$marks['###FORMHEADER###'] = $this->pi_getLL('formHeader');
				$marks['###FORMSTART###'] = '<form method="post" action="' . $this->pi_getPageLink($GLOBALS['TSFE']->id) . '">';
				$marks['###CITYV###'] = htmlspecialchars($key);
				$marks['###RADIUSV###'] = (int) $this->_GP['radius'];
				$marks['###COUNTRYV###'] = htmlspecialchars($this->_GP['country']);
				$marks['###LATV###'] = (float) $lat[$i];
				$marks['###LONV###'] = (float) $lon[$i];
				$marks['###STORESEARCH###'] = $this->pi_getLL('storeSearch');

				$i++;
				$final .= $this->cObj->substituteMarkerArray($subpart, $marks);

			}
			$final .= $this->cObj->getSubpart($this->template, "###SEARCHCITY_FOOTER###");
			return $final;
		}

		$HTTP_POST_VARS['lat_lon'] = $center_lat_lon;

		$this->_GP['lat_lon'] = $center_lat_lon;
		/*
		 * get the stores
		 */
		$mysqlResults = $this->inradius($this->_GP, (int) $this->_GP['radius']);

		if ($mysqlResults == NULL) {
			return '<div class="searchResultHeader">' . $this->pi_getLL('noStoresFound') . '</div>';
		}

		$center_lat = $center_lat_lon->lat;
		$center_lon = $center_lat_lon->lon;
		$center_lat_float = floatval($center_lat);
		$center_lon_float = floatval($center_lon);
		$content .= '<div id="results" class="tx_locator_results">';

		if ($this->conf['showResultTable']) {
			if ($mysqlResults == NULL || mysql_num_rows($mysqlResults) == 0) {
				$marksHeader['###TITLE###'] = $this->pi_getLL('theSearchFrom') .
				$this->_GP['numberOfStores'] . $this->pi_getLL('stores') . $this->pi_getLL('inRadius') .
				(int) $this->_GP['radius'] . ' ' . $this->pi_getLL($this->conf['distanceUnit']) . ' ' . $this->pi_getLL('gaveNoResults');

				$marksHeader['###SEARCHRESULT_HEADER###'] = $this->pi_getLL('formHeader');
				$subpart = $this->cObj->getSubpart($this->template, "###SEARCHMAILRESULT_HEADER###");
				$final .= $this->cObj->substituteMarkerArray($subpart, $marksHeader);
			} else {
				$marksHeader['###TITLE###'] = $this->pi_getLL('searchFor') . $this->_GP['address'] . '&nbsp;' . $this->_GP['zipcode'] .
				'&nbsp;' . $this->_GP['city'] . " (" . $this->_GP['country'] . ")";
				if ($this->conf['showOnlyMailResultTable'])
					$subpart = $this->cObj->getSubpart($this->template, "###SEARCHONLYMAILRESULTTABLE_HEADER###");
				else
					$subpart = $this->cObj->getSubpart($this->template, "###SEARCHMAILRESULT_HEADER###");
				$final .= $this->cObj->substituteMarkerArray($subpart, $marksHeader);

				$marks['###HIDDEN###'] = '<input type="hidden" name="tx_locator_pi1[firstname]" value="' .
				$this->_GP['firstname'] . '">' .
				'<input type="hidden" name="tx_locator_pi1[surname]" value="' . htmlspecialchars($this->_GP['surname']) . '">' .
				'<input type="hidden" name="tx_locator_pi1[address]" value="' . htmlspecialchars($this->_GP['address']) . '">' .
				'<input type="hidden" name="tx_locator_pi1[zipcode]" value="' . htmlspecialchars($this->_GP['zipcode']) . '">' .
				'<input type="hidden" name="tx_locator_pi1[city]" value="' . htmlspecialchars($this->_GP['city']) . '">' .
				'<input type="hidden" name="tx_locator_pi1[country]" value="' . htmlspecialchars($this->_GP['country']) . '">' .
				'<input type="hidden" name="tx_locator_pi1[radius]" value="' . (int) $this->_GP['radius'] . '">' .
				'<input type="hidden" name="tx_locator_pi1[phone]" value="' . htmlspecialchars($this->_GP['phone']) . '">' .
				'<input type="hidden" name="tx_locator_pi1[email]" value="' . htmlspecialchars($this->_GP['email']) . '">' .
				'<input type="hidden" name="tx_locator_pi1[mode]" value="sendMails">';

				$marks['###ADDITIONALDATA###'] = $this->pi_getLL('additionalData');

				$marks['###FORM###'] = '<form method="post" action="' . $this->pi_getPageLink($GLOBALS['TSFE']->id) . '">';
				$marks['###ELEVATOR###'] = $this->pi_getLL('elevator');
				$marks['###ELEVATORV###'] = '<input type="text" name="tx_locator_pi1[elevator]" value="' . htmlspecialchars($this->_GP['elevator']) . '"/>';
				$marks['###FLOOR###'] = $this->pi_getLL('floor');
				$marks['###FLOORV###'] = '<input type="text" name="tx_locator_pi1[floor]" value="' . htmlspecialchars($this->_GP['floor']) . '"/>';
				$marks['###NOBEDROOMS###'] = $this->pi_getLL('noBedRooms');
				$marks['###NOBEDROOMSV###'] = '<input type="text" name="tx_locator_pi1[noBedRooms]" value="' . htmlspecialchars($this->_GP['noBedRooms']) . '"/>';

				$marks['###MOVINGTO###'] = $this->pi_getLL('movingTo');
				$marks['###TOADDRESS###'] = $this->pi_getLL('toAddress');
				$marks['###TOADDRESSV###'] = '<input type="text" name="tx_locator_pi1[toAddress]" value="' . htmlspecialchars($this->_GP['toAddress']) . '"/>';
				$marks['###TOZIPCODE###'] = $this->pi_getLL('toZipcode');
				$marks['###TOZIPCODEV###'] = '<input type="text" name="tx_locator_pi1[toZipcode]" value="' . htmlspecialchars($this->_GP['toZipcode']) . '"/>';
				$marks['###TOCITY###'] = $this->pi_getLL('toCity');
				$marks['###TOCITYV###'] = '<input type="text" name="tx_locator_pi1[toCity]" value="' . htmlspecialchars($this->_GP['toCity']) . '"/>';
				$marks['###TOELEVATOR###'] = $this->pi_getLL('toElevator');
				$marks['###TOELEVATORV###'] = '<input type="text" name="tx_locator_pi1[toElevator]" value="' . htmlspecialchars($this->_GP['toElevator']) . '"/>';
				$marks['###TOFLOOR###'] = $this->pi_getLL('toFloor');
				$marks['###TOFLOORV###'] = '<input type="text" name="tx_locator_pi1[toFloor]" value="' . htmlspecialchars($this->_GP['toFloor']) . '"/>';
				$marks['###TOPACKING###'] = $this->pi_getLL('toPacking');
				$marks['###TOPACKINGV###'] = '<input type="text" name="tx_locator_pi1[toPacking]" value="' . htmlspecialchars($this->_GP['toPacking']) . '"/>';
				$marks['###TOKINDOFMOVEMENT###'] = $this->pi_getLL('toKindOfMovement');
				$marks['###TOKINDOFMOVEMENTV###'] = '<input type="text" name="tx_locator_pi1[toKindOfMovement]" value="' . htmlspecialchars($this->_GP['toKindOfMovement']) . '"/>';
				$marks['###TOSTORAGE###'] = $this->pi_getLL('toStorage');
				$marks['###TOSTORAGEV###'] = '<input type="text" name="tx_locator_pi1[toStorage]" value="' . htmlspecialchars($this->_GP['toStorage']) . '"/>';
				$marks['###TOMOVINGDATE###'] = $this->pi_getLL('toMovingDate');
				$marks['###TOMOVINGDATEV###'] = '<input type="text" name="tx_locator_pi1[toMovingDate]" value="' . htmlspecialchars($this->_GP['toMovingDate']) . '"/>';
				$marks['###TOCOMMENTS###'] = $this->pi_getLL('toComments');
				$marks['###TOCOMMENTSV###'] = '<textarea cols="40" rows="4" name="tx_locator_pi1[toComments]"/>' . htmlspecialchars($this->_GP['toComments']) . '</textarea>';

				$marks['###MAILBUTTON###'] = '<input type="submit" name="tx_locator_pi1[mailButton]" value="' .
				$this->pi_getLL('mailButtonLabel') . '"/>';

				$counter = 0;
				while (($rows = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($mysqlResults)) && ($counter < $this->conf['numberOfMails'])) {
					$counter++;
					$lat = htmlspecialchars($rows['lat']);
					$lon = htmlspecialchars($rows['lon']);
					$address = htmlspecialchars($rows['address']);
					$city = htmlspecialchars($rows['city']);
					$state = htmlspecialchars($rows['state']);
					$zipcode = htmlspecialchars($rows['zipcode']);
					$country = htmlspecialchars($rows['country']);
					$storename = htmlspecialchars($rows['storename']);
					$company = htmlspecialchars($rows['company']);
					$phone = htmlspecialchars($rows['phone']);
					$hours = htmlspecialchars($rows['hours']);

					$url = htmlspecialchars($rows['url']);
					if ($url == 'http://')
						$url = '';

					//$image      = $rows['imageurl'];
					$icon = htmlspecialchars($rows['icon']);
					$notes = htmlspecialchars($rows['notes']);
					$phone = htmlspecialchars($rows['phone']);
					$id = htmlspecialchars($rows['id']);
					$image = htmlspecialchars($rows['imageurl']);
					$distance = number_format($rows['distance'], 2, '.', '');

					if ($url > '') {
						$link = $this->cObj->getTypoLink_URL($url);
						$marks['###STORENAMEV###'] = '<a href="' . $link . '" target="_blank">' . $storename . '</a>';
						$marks['###COMPANYV###'] = '<a href="' . $link . '" target="_blank">' . $company . '</a>';
					} else {
						$marks['###STORENAMEV###'] = $storename;
						$marks['###COMPANYV###'] = $company;
						$link = '';
					}
					$marks['###ADDRESSV###'] = $address;
					$marks['###ZIPCODEV###'] = $zipcode;
					$marks['###CITYV###'] = $city;
					$marks['###PHONEV###'] = $phone;
					$marks['###HOURS###'] = nl2br($hours);
					$marks['###URLV###'] = '<a href="' . $link . '" target="_blank">' . $link . '</a>';

					if ($this->conf['distanceUnit'] == 'km')
						$marks['###DISTANCEV###'] = $distance . ' km';
					else
						$marks['###DISTANCEV###'] = number_format($distance / 1.6, 2, '.', '') . ' miles';

					if ($this->conf['showStoreImage']) {
						$marks['###IMAGEV###'] = ($image > '') ? '<td><img src="uploads/tx_locator/' .
						$image . '"/></td>' : '<td></td>';
					} else
						$marks['###IMAGEV###'] = '';
					if ($this->conf['showOnlyMailResultTable'])
						$part = $this->cObj->getSubpart($this->template, "###SEARCHONLYMAILRESULTTABLE_DATA###");
					else
						$part = $this->cObj->getSubpart($this->template, "###SEARCHMAILRESULT_DATA###");
					$final .= $this->cObj->substituteMarkerArray($part, $marks);
				}
			} // end else
			if ($this->conf['showOnlyMailResultTable'])
				$part = $this->cObj->getSubpart($this->template, "###SEARCHONLYHMAILRESULTTABLE_FOOTER###");
			else
				$part = $this->cObj->getSubpart($this->template, "###SEARCHMAILRESULT_FOOTER###");
			$final .= $this->cObj->substituteMarkerArray($part, $marks);
		} // end if ($this->conf['displayResultTable'])

		$listContent .= "</table></div>";

		return $final;

	}

	function cleanHTML($html) {
		// this function isn't used yet
		$doc = new DOMDocument();
		// load the HTML string we want to strip
		$doc->loadHTML($html);
		// get all the script tags
		$script_tags = $doc->getElementsByTagName('script');
		$length = $script_tags->length;
		// for each tag, remove it from the DOM
		for ($i = 0; $i < $length; $i++) {
		  $script_tags->item($i)->parentNode->removeChild($script_tags->item($i));
		}
		// get the HTML string back
		$clean = $doc->saveHTML();
		return $clean;

	}


}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/locator/pi1/class.tx_locator_model.php']) {
	include_once ($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/locator/pi1/class.tx_locator_model.php']);
}
?>